<?php
	/**
	 * @property NetsuiteConnection $netsuite_connection
	 */
	class wc_create_update_product_translation
	{
		public $netsuite_connection = null;
		
		public function __construct()
		{
			// hook for when a product is created from api
			// hook for when a product is created from api
			add_action('woocommerce_api_create_product', [$this, 'api_product'], 10, 2);
			// hook for when a product is update from api
			add_action('woocommerce_api_edit_product', [$this, 'api_product'], 10, 2);
			
			add_action('in8_endpoint_meta_post_after',[$this, 'meta_post'], 10, 2);
			add_action('in8_endpoint_meta_post_after_update',[$this, 'meta_post_single'], 10, 2);
			
			add_action('in8_translations_product_variations', [$this, 'async_variations'], 10, 3);
			add_action('in8_move_variations_to_translations', [$this, 'move_variations_to_translations'], 10, 2);
			
			
		}
		
		public function netsuite_init($post_id)
		{
			
			if (!class_exists('NetsuiteConnection')) { //in case of ajax
				//netsuite api connection file
				require_once(WP_PLUGIN_DIR . '/in8sync/code/netsuiteconnection.php');
			}
			
			//netsuite connection object
			$this->netsuite_connection = new NetsuiteConnection($post_id, 'product wpml');
		}
		
		public function api_product($id, $data)
		{
			if ( ! isset($data['translations'])) return;
			if( isset($data['translation_of']) ) return;
			
			
			global $wpdb, $sitepress, $woocommerce_wpml;
			$translations = $data['translations'];
			
			$this->netsuite_init($id);
			
			$translations = apply_filters('in8_product_sync_wpml_trans_data', $translations, $id, $data);
			
			$this->netsuite_connection->processMessage('Product wpml translations sync start!');
			
			
			// get the languages as a string
			$langs = implode(', ', array_filter(array_map(function($a){ return @$a['locale']; }, $translations)));
			$this->netsuite_connection->processMessage("We are going to create/update translations for <strong>{$langs}</strong>.");
			
			// the original product will be updated here so we need to make sure we're not sending it to netsuite right here
			add_filter('in8_product_sync_skip', [$this, 'skip_product'], 10, 2);
			foreach ($translations as $info) {
				$lang = $info['locale'];
				$lang = $this->get_lang_code($lang);
				
				// check if we want to continue
				if( apply_filters('in8_product_sync_wpml_trans_skip', false, $info, $lang, $id, $data) ){
					continue;
				}
				
				
				if( $lang == false ){
					// TODO error notification this language it's not enabled
					$this->netsuite_connection->processMessage("<strong>Language `{$info['locale']}` is not set up on woocommerce.</strong>", false, true);
					continue;
				}
//				// we don't need to create another for default language
				$default_language = $sitepress->get_default_language();
				if( $lang == $default_language ) continue;
				
				$translation_id = apply_filters( 'wpml_object_id', $id, 'product', false, $lang );
				
				// new way
				$new_lang_data = $data;
				$new_lang_data['lang'] = $lang;
				$new_lang_data['translation_of'] = $id;
				$new_lang_data['title'] = !empty($info['displayname']) ? $info['displayname'] : $data['title'];
				$new_lang_data['description'] = $info['storedetaileddescription'];
				unset($new_lang_data['variations']);
				unset($new_lang_data['translations']);
//				unset($new_lang_data['attributes']);
				if( empty($new_lang_data['title']) ){
					// TODO error notification this language it's not enabled
					$this->netsuite_connection->processMessage("<strong>Language `{$info['locale']}` doesn't have a title</strong>", false, true);
					continue;
				}
				
				if( !empty($new_lang_data['id']) && !empty($translation_id) ){
					$new_lang_data['id'] = $translation_id;
					try {
						WC()->api->WC_API_Products->edit_product($translation_id, ['product' => $new_lang_data]);
					} catch ( WC_API_Exception $e ) {
						echo "<pre>";print_r(new WP_Error( $e->getErrorCode(), $e->getMessage(), array( 'status' => $e->getCode() ) ));echo "</pre>";
					}
				}else{
					$sku = $new_lang_data['sku'];
					unset($new_lang_data['sku']);
					
					try {
						WC()->api->WC_API_Products->create_product(['product' => $new_lang_data]);
					} catch ( WC_API_Exception $e ) {
						echo "<pre>";print_r(new WP_Error( $e->getErrorCode(), $e->getMessage(), array( 'status' => $e->getCode() ) ));echo "</pre>";
					}
				}
				if( !empty($translation_id) ) {
					update_post_meta( $translation_id, '_sku', $sku );
					// TODO - remove
					// handle variations translation
//					if( IN8_TRANSLATION_VARIATION_ASYNC === FAlSE ) {
//						$sync_variations_data = new WCML_Synchronize_Variations_Data( $woocommerce_wpml, $sitepress, $wpdb );
//						$sync_variations_data->sync_product_variations( $id, $translation_id, $lang );
//					}else{
//						wp_schedule_single_event(time() + 60, 'in8_translations_product_variations', [$id, $translation_id, $lang]);
//					}
					echo "<pre>";print_r($translation_id);echo "</pre>";
					$this->unique_product_attributes($translation_id);
				}
				
				// TODO - remove
				// OLD algorithm - does NOT support variable translations
//				$infoPost = [
//					'post_title'   => $info['displayname'],
//					'post_content' => $info['storedetaileddescription'],
//					'post_content' => !empty($info['storedetaileddescription']) ? $info['storedetaileddescription'] : ' ',
//					'post_excerpt' => !empty($info['storedescription']) ? $info['storedescription'] : ' ',
//					'post_type'    => 'product',
//					'post_status'  => 'publish',
//				];
//
//				$infoPost = apply_filters('in8_product_sync_wpml_trans_post_data', $infoPost, $translation_id); // translation_id will be false on create
//
//				if( !$translation_id ) {
//					$translation_id = wp_insert_post($infoPost);
//				}else{
//					$infoPost['ID'] = $translation_id;
//					wp_update_post($infoPost);
//				}
//
//				if( !$translation_id ){
//					// TODO error notification the translation product couldn't create
//					$this->netsuite_connection->processMessage("<strong>The translation for language `{$info['locale']}` couldn't be created</strong>", false, true);
//					continue;
//				}
//
//				// featured image
//				$attachment_id = get_post_thumbnail_id($id);
//				if( $attachment_id ) {
//					set_post_thumbnail($translation_id, $attachment_id);
//				}
//
//				// Collect translation informations
//				$get_language_args = ['element_id' => $id, 'element_type' => 'product'];
//				$wpml_element_type = apply_filters('wpml_element_type', 'product');
//				$original_post_language_info = apply_filters('wpml_element_language_details', NULL, $get_language_args);
//				$set_language_args = [
//					'element_id'           => $translation_id,
//					'element_type'         => $wpml_element_type,
//					'trid'                 => $original_post_language_info->trid,
//					'language_code'        => $lang,
//					'source_language_code' => $original_post_language_info->language_code,
//				];
//
//
////				do_action('wpml_set_element_language_details', $set_language_args);
//				$trid = $sitepress->set_element_language_details($set_language_args['element_id'], $set_language_args['element_type'], $set_language_args['trid'], $set_language_args['language_code'], $set_language_args['source_language_code']);
//				if( empty($trid) ){
//					// TODO error notification translation couldn't be created
//					$this->netsuite_connection->processMessage("<strong>The translation was created but couldn't be setup, id# {$translation_id}</strong>", false, true);
//					continue;
//				}
//				$job_id = $this->get_job_id( $trid, $translation_id, $original_post_language_info->language_code, $lang );
//				if( empty($job_id) ){
//					// TODO error notification we couldn't set the translation as complete
//					$this->netsuite_connection->processMessage("<strong>The translation was created but the job couldn't be created, id# {$translation_id}</strong>", false, true);
//					continue;
//				}
//				do_action('in8_product_sync_wpml_trans_saved', $trid);
				
				$this->netsuite_connection->processMessage("Translation created/updated for <strong>{$info['locale']}</strong>");
				
			}
			// remove the filter so we can use it latter
			remove_filter('in8_product_sync_skip', [$this, 'skip_product'], 10, 2);
			$this->netsuite_connection->processMessage("Product wpml translations sync completed!", true);
		}
		
		public function meta_post($data){
			
			if( empty($data) ) return;

//			// Copy variations from main to translations
//			foreach($data as $productFields){
//				// only need it for the first variation item
//				$this->move_variations_to_translations($productFields['meta'], $productFields['id']);
//				break;
//			}
//			// END
			
			foreach($data as $productFields){
				if( !empty($productFields['meta']['translations']) ){
					$this->meta_post_single($productFields['meta'], $productFields['id']);
				}
			}
			
		}
		public function meta_post_single($data, $id){
			if ( ! isset($data['translations'])) return;
			global $sitepress;
			
			
			// in case this is a variation, we want to move the variations from main product to translations
//			$this->move_variations_to_translations($data, $id);
			$this->schedule_move_variations($data, $id);
//			wp_schedule_single_event( time() + 300, 'in8_move_variations_to_translations', [
//				$data,
//				$id,
//			] );
			
			// only update the products and not variations
//			if( get_post_type($id) != 'product' ) return;
			
			foreach ($data['translations'] as $info) {
				$translationData = $data;
				$lang = $info['locale'];
				$lang = $this->get_lang_code($lang);
				
				// check if we want to continue
				if( apply_filters('in8_product_sync_wpml_trans_skip', false, $info, $lang, $id, $translationData) ){
					continue;
				}
				
				
				if( $lang == false ){
					// TODO error notification this language it's not enabled
					continue;
				}
//				// we don't need to create another for default language
				$default_language = $sitepress->get_default_language();
				if( $lang == $default_language ) continue;
				
				
				$translation_id = apply_filters( 'wpml_object_id', $id, 'product', false, $lang );
				if( empty($translation_id) ) continue;
				
				
				
				// copy images from main but don't reupload
				foreach ( $translationData as $key => $val ) {
					if ( preg_match( '/alt_image_/i', $key ) ) {
						$alt_images = get_post_meta( $id, apply_filters('in8_product_additional_variation_images', '_wc_additional_variation_images'), TRUE );
						update_post_meta($translation_id, apply_filters('in8_product_additional_variation_images', '_wc_additional_variation_images'), $alt_images);
						unset($translationData[$key]);
					}
					if ( $key == 'image_main' ) {
						$existing_image_main = get_post_thumbnail_id( $id );
						set_post_thumbnail( $translation_id, $existing_image_main );
						unset($translationData[$key]);
					}
					if ( preg_match( '/image_gallery/i', $key ) ) {
						$existing_gallery_images = get_post_meta( $id, '_product_image_gallery', TRUE );
						update_post_meta( $translation_id, '_product_image_gallery', $existing_gallery_images );
						unset($translationData[$key]);
					}
				}
				
				WC()->api->meta_post->process_meta_post_single($translationData, $translation_id);
				
				// TODO - remove
//				$this->async_variations($id, $translation_id, $lang);
//				if( get_post_type($id) == 'product' ) {
//					if ( empty( wp_next_scheduled( 'in8_translations_product_variations', [
//						$id,
//						$translation_id,
//						$lang
//					] ) ) ) {
//						wp_schedule_single_event( time() + 120, 'in8_translations_product_variations', [
//							$id,
//							$translation_id,
//							$lang
//						] );
//					} else {
//						wp_clear_scheduled_hook( 'in8_translations_product_variations', [
//							$id,
//							$translation_id,
//							$lang
//						] );
//						wp_schedule_single_event( time() + 120, 'in8_translations_product_variations', [
//							$id,
//							$translation_id,
//							$lang
//						] );
//					}
//				}
			
			}
		}
		
		public function schedule_move_variations($data, $variation_id){
			if ( empty( wp_next_scheduled( 'in8_move_variations_to_translations', [
				$data,
				$variation_id,
			] ) ) ) {
				wp_schedule_single_event( time() + 120, 'in8_move_variations_to_translations', [
					$data,
					$variation_id,
				] );
			} else {
				wp_clear_scheduled_hook( 'in8_move_variations_to_translations', [
					$data,
					$variation_id,
				] );
				wp_schedule_single_event( time() + 120, 'in8_move_variations_to_translations', [
					$data,
					$variation_id,
				] );
			}
		}
		public function move_variations_to_translations($data, $variation_id){
			
			if( get_post_type($variation_id) != 'product_variation' ) return;
			
			global $sitepress;
			
			if( !empty($data['translations']) ){
				foreach($data['translations'] as $info){
					$lang = $info['locale'];
					$lang = $this->get_lang_code($lang);
					$product_id = wp_get_post_parent_id($variation_id);
					
					// check if we want to continue
					if( apply_filters('in8_product_sync_wpml_trans_skip', false, $info, $lang, $product_id, $data) ){
						continue;
					}
					
					
					if( $lang == false ){
						// TODO error notification this language it's not enabled
						continue;
					}
					
					$default_language = $sitepress->get_default_language();
					if( $lang == $default_language ) continue;
					
					$translation_id = apply_filters( 'wpml_object_id', $product_id, 'product', false, $lang );
					if( empty($translation_id) ) continue;
					if( $translation_id == $product_id ) return;
					
					$this->async_variations($product_id, $translation_id, $lang);
				}
			}
		}
		public function async_variations($id, $translation_id, $lang){
			global $woocommerce_wpml, $sitepress, $wpdb;

//			$sync_variations_data = new WCML_Synchronize_Variations_Data( $woocommerce_wpml, $sitepress, $wpdb );
//			$sync_variations_data->sync_product_variations( $id, $translation_id, $lang );
			$woocommerce_wpml->sync_product_data->sync_product_data($id, $translation_id, $lang);
		}
		
		
		public function unique_product_attributes($id){
			/* @var WC_Product $product */
			$product = wc_get_product($id);
			$attributes = $product->get_attributes();
			foreach($attributes as $key => $attribute){
				/* @var WC_Product_Attribute $attribute */
				$options = $attribute->get_options();
				if( array_filter($options, 'is_int') ) {
					$attribute->set_options( array_unique( $attribute->get_options() ) );
					$attributes[$key] = $attribute;
				}
			}
			$product->set_attributes($attributes);
		}
		
		/**
		 * Always skip the products when we're updating them to create translations
		 *
		 * @param $skip
		 * @param $id
		 *
		 * @return bool
		 */
		function skip_product($skip, $id)
		{
			return true;
		}
		
		/**
		 * Search the translation status table to find if there's there is already a translation and return it
		 *
		 * @param $trid
		 *
		 * @return bool
		 */
		function get_translation_status($trid){
			global $wpdb;
			$status = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}icl_translation_status WHERE translation_id={$trid} LIMIT 1");
			if( isset($status[0]) ){
				return $status[0];
			}
			return false;
		}
		
		/**
		 * Get the job if exists based on the status
		 *
		 * @param $status
		 *
		 * @return bool
		 */
		function get_job_from_status($status){
			if( !isset($status->rid) ) return false;
			
			global $wpdb;
			$job = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}icl_translate_job WHERE rid={$status->rid} LIMIT 1");
			if( isset($job[0]) ){
				return $job[0];
			}
			return false;
		}
		
		/**
		 * Set the translation status as completed
		 *
		 * @param $status
		 *
		 * @return bool
		 */
		const COMPLETE_STATUS_CODE = 10;
		function translation_complete($status){
			if( !isset($status->rid) ) return false;
			
			global $wpdb;
			$wpdb->update("{$wpdb->prefix}icl_translation_status", ['status' => self::COMPLETE_STATUS_CODE, 'needs_update' => 0], ['rid' => $status->rid]);
			return true;
		}
		
		/**
		 * Get job id if exists
		 *
		 * @param $trid
		 * @param $post_id
		 * @param $source_language_code
		 * @param $language_code
		 *
		 * @return bool
		 */
		function get_job_id($trid, $post_id, $source_language_code, $language_code){
			global $iclTranslationManagement, $wpml_post_translations, $wpml_translation_job_factory, $sitepress;
			$job_id = false;
			if ( $trid && $language_code ) {
				
				// Get translation status row, if this exists then it's an update so we don't need to create the job
				$status = $this->get_translation_status($trid);
				if( isset($status->rid) ) {
					$job_id = $this->get_job_from_status($status);
				}
				
				// The job doesn't exist yet we want to make sure it's not been created
				if ( ! $job_id ) {
					$job_id = $iclTranslationManagement->get_translation_job_id( $trid,
						$language_code );
					if ( ! $job_id ) {
						$blog_translators = wpml_tm_load_blog_translators();
						$args             = array(
							'lang_from' => $source_language_code,
							'lang_to'   => $language_code,
							'job_id'    => $job_id
						);
						if ( $post_id && $blog_translators->is_translator( $sitepress->get_current_user()->ID,
								$args )
						) {
							// Create the job
							$job_id = $wpml_translation_job_factory->create_local_post_job( $post_id,
								$language_code );
						}
					}
				}
			}
			if( !$status ){
				$status = $this->get_translation_status($trid);
				// Mark this product as completed
				$this->translation_complete($status);
			}
			return $job_id;
		}
		
		/**
		 * Get language code conversion from netsuite to wpml
		 *
		 * @param $code
		 *
		 * @return bool
		 */
		function get_lang_code($code){
			global $sitepress;
			$active_langs = $sitepress->get_active_languages();
			foreach($active_langs as $lang){
				if( $lang['default_locale'] == $code || $lang['code'] == $code ){
					return $lang['code'];
				}
			}
			return false;
		}
		
	}
	
	new wc_create_update_product_translation();