<?php
	
	// If the transaction id is missing set the order to sync async
	class In8_Products_Options {
		public function __construct() {
			
			$this->show_internal_id_filter();
			
			$this->api_search_by_sku_include_draft();
			
			$this->handle_product_type_change();
			
			$this->show_product_cat_internal_id();
			
			$this->handle_shipping_class_mapping();
		}
		
		
		function show_internal_id_filter() {
			
			add_filter( 'views_edit-product', function ( $views ) {
				
				$count = "(" . In8_Products_Options::getUnSyncedCount() . ")";
				$field = '<a href="' . admin_url( 'edit.php?post_type=product&un-synced=1' ) . '">Unsynced Products ' . $count . '</a>';
				if ( isset( $_REQUEST['un-synced'] ) ) {
					$field = 'Unsynced Products ' . $count;
				}
				
				$views['un-synced'] = $field;
				
				return $views;
				
			} );
			add_filter( 'pre_get_posts', function ( $query ) {
				global $pagenow;
				
				if ( 'edit.php' != $pagenow || ! $query->is_admin ) {
					return $query;
				}
				
				if ( isset( $_REQUEST['un-synced'] ) && isset($_REQUEST['post_type']) && $_REQUEST['post_type'] == 'product' ) {
					/* @var WP_Query */
					$meta_query = [
						[
							'key'     => 'internalid',
							'compare' => 'NOT EXISTS',
						],
					];
					
					$query->set( 'post_status', 'publish' );
					$query->set( 'meta_query', $meta_query );
				}
				
				return $query;
			} );
		}
		
		static function getUnSyncedCount() {
			$args = [
				'meta_key'       => 'internalid',
				'posts_per_page' => - 1,
				'meta_compare'   => "NOT EXISTS",
				'post_type'      => 'product',
				'post_status'    => 'publish',
				'fields'         => 'ids',
			];
			
			$wp_query = new WP_Query( $args );
			
			return $wp_query->post_count;
		}
		
		function api_search_by_sku_include_draft(){
			add_filter('woocommerce_api_query_args', function($default){
				
				if( isset($_REQUEST['filter']) && isset($_REQUEST['filter']['sku']) ) {
					$default['post_status'] = [ 'publish', 'draft' ];
				}
				
				return $default;
			});
			
			add_filter('woocommerce_variable_children_args', function($args){
				if( isset($_REQUEST['ns']) ){
					if( !is_array($args['post_status']) ){
						$args['post_status'] = [$args['post_status']];
					}
					$args['post_status'][] = 'draft';
				}
				
				return $args;
			});
		}
		
		
		
		
		/**
		 * This function allows us to change the product type by sending _in8_product_type on the main products endpoint
		 */
		function handle_product_type_change(){
			add_filter('woocommerce_api_dispatch_args', function($args){
				
				if( strpos($args['_route'], '/products/') !== FALSE && !empty($args['id']) ){
					if( !empty($args['data']) && !empty($args['data']['product']['_in8_product_type']) ){
						$product = wc_get_product($args['id']);
						if( $args['data']['product']['_in8_product_type'] !== $product->get_type() ) {
							$classname = WC_Product_Factory::get_classname_from_product_type( $args['data']['product']['_in8_product_type'] );
							
							if ( ! class_exists( $classname ) ) {
								$classname = 'WC_Product_Simple';
							}
							
							$product = new $classname( $args['id'] );
							$product->save();
						}
					}
				}
				
				return $args;
			});
			
		}
		
		
		public function show_product_cat_internal_id(){
			
			
			add_action('product_cat_edit_form_fields', function ($tag){
				$internal_id = get_term_meta($tag->term_id, 'internalid', true);
				?>
				<tr class="form-field term-name-wrap">
					<th scope="row"><label for="internalid"><?php _ex( 'NS Internal ID', 'term internal id' ); ?></label></th>
					<td><input name="internalid" id="internalid" type="text" value="<?= $internal_id ?>" />
				</tr>
				<?php
			});
			add_action('edited_product_cat', 'in8_save_term_ns_id', 10, 2);
			add_action('create_product_cat', 'in8_save_term_ns_id', 10, 2);
			function in8_save_term_ns_id($term_id){
				if( isset($_REQUEST['internalid']) ) {
					update_term_meta( $term_id, 'internalid', $_REQUEST['internalid'] );
				}
			}
			
			add_filter( 'manage_edit-product_cat_columns', function($default){
				$default['internalid'] = 'NS ID';
				return $default;
			} );
			add_filter( 'manage_product_cat_custom_column', function($columns, $column, $id){
				
				if( $column == 'internalid' ){
					$columns .= get_term_meta($id, 'internalid', true);
				}
				
				return $columns;
			}, 10, 3 );
			add_filter( 'manage_edit-product_cat_columns', function($default){
				$default['wcid'] = 'WC ID';
				return $default;
			} );
			add_filter( 'manage_product_cat_custom_column', function($columns, $column, $id){
				
				if( $column == 'wcid' ){
					$columns .= $id;
				}
				
				return $columns;
			}, 10, 3 );
		}
		
		
		public function handle_shipping_class_mapping(){
			add_filter('in8_endpoint_meta_post_data', function($data, $id){
				if( !empty($data['shipping_class']) ){
					$slug = apply_filters('in8_product_shipping_class_mapping', $data['shipping_class'], $id);
					$shipping_class = get_term_by('slug', $slug, 'product_shipping_class');
					if( empty($shipping_class) || is_wp_error($shipping_class) ){
						$shipping_class_id = -1;
					}else {
						$shipping_class_id = $shipping_class->term_id;
					}
					
					$product = wc_get_product( $id );
					
					$product->set_shipping_class_id( $shipping_class_id );
					
					$product->save();
				}
				return $data;
			}, 10, 2);
		}
		
	}
	
	new In8_Products_Options();